from photonai_graph.GraphEmbedding.graph_embedding_base import GraphEmbeddingBase
from photonai_graph.GraphEmbedding.graph_embedding_hope import GraphEmbeddingHOPE
from photonai_graph.GraphEmbedding.graph_embedding_laplacian_eigenmaps import GraphEmbeddingLaplacianEigenmaps
from photonai_graph.GraphEmbedding.graph_embedding_locally_linear_embedding import GraphEmbeddingLocallyLinearEmbedding
